package com.hzs.scm.trans.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.hzs.common.domain.scm.trans.IcTransInH;
import com.hzs.common.domain.scm.trans.ext.IcTransInHExt;
import com.hzs.common.domain.scm.trans.ext.IcTransOutHExt;
import com.hzs.common.domain.scm.trans.ext.TransferOrderExt;
import com.hzs.scm.pub.param.ScmApproveParam;
import com.hzs.scm.trans.param.TransferParam;
import com.hzs.system.sys.dto.ApprovalBusinessResultDTO;
import com.hzs.system.sys.dto.LoginUser;

import java.util.List;

/**
 * <p>
 * 调拨入库单表头 服务类
 * </p>
 *
 * @author hzs
 * @since 2023-12-04
 */
public interface IIcTransInHService extends IService<IcTransInH> {

    /*
     * @description: 新增保存采购入库单
     * @author: sui q
     * @date: 2023/11/22 9:56
     * @param: null null
     **/
    void insertSaveTransferInByTransferOrder(List<IcTransOutHExt> transOutExtList, TransferParam transferParam, Long pkMember);

    /*
     * @description: 修改保存采购入库单
     * @author: sui q
     * @date: 2023/11/23 9:10
     * @param: null null
     **/
    void updateSaveTransferInByPoOrder(List<IcTransOutHExt> transOutExtList, IcTransInHExt transInExt,
                                       TransferParam transferParam, Long pkMember);

    /*
     * @description: 根据主键删除采购入库单，只有保存的可以删除
     * @author: sui q
     * @date: 2023/11/23 11:22
     * @param: null null
     **/
    void deleteTransferInByPk(Long pkTransferIn, Long pkMember);

    /*
     * @description: 根据主键查询入库单
     * @author: sui q
     * @date: 2023/11/24 13:52
     * @param: null null
     **/
    IcTransInHExt queryTransferInInhExtById(Long pkId, Integer orderStatus, Integer pkCountry);

    /*
     * @description: 查询会员端显示的采购入库单
     * @author: sui q
     * @date: 2023/11/25 9:55
     * @param: null null
     **/
    IcTransInHExt queryDisTransferInInhExtById(Long pkId, Integer orderStatus);

    /*
     * @description: 查询列表
     * @author: sui q
     * @date: 2023/11/28 15:48
     * @param: null null
     **/
    List<IcTransInHExt> queryDisTransferInInExtByCondition(TransferParam transferParam);
}
